fn main() {
    use std::fs::File;
    use std::io::prelude::*;

    const MAGAZINE :usize     = 0x100;
    const MAGAZINE_ROW :usize = 0x101;

    // define schema "Magazine Row"
    szun::Schema::with(vec![
        ("Content", szun::natural()),
        ("Quantity", szun::natural()),
    ]).bind(MAGAZINE_ROW);

    // define schema "Magazine"
    let magazine = szun::Schema::with(vec![
        ("Capacity", szun::natural()),
        ("Quantity", szun::natural()),
        ("Content", szun::list(szun::record(MAGAZINE_ROW))),
    ]);
    magazine.bind(MAGAZINE);

    // create record "Magazine"
    let _data = szun::Record::with(MAGAZINE, vec![
        ("Capacity", *szun::Natural::with(30)),
        ("Quantity", *szun::Natural::with(25)),
        ("Content", *szun::List::with(szun::record(MAGAZINE_ROW), vec![
            *szun::Record::with(MAGAZINE_ROW, vec![
                ("Content", *szun::Natural::with(15)),
                ("Quantity", *szun::Natural::with(5)),
            ]).unwrap(),
            *szun::Record::with(MAGAZINE_ROW, vec![
                ("Content", *szun::Natural::with(16)),
                ("Quantity", *szun::Natural::with(20)),
            ]).unwrap(),
        ])),
    ]).unwrap();

    // encode record
    let out = szun::encode(*magazine);

    // write encoding to file
    let mut file = File::create("target/dat.szn").unwrap();
    file.write_all(&out).ok();

    // print hex series of encoding
    print!("[{}]: ", out.len());
    for byte in &out {
        print!("{:02x} ", *byte);
    } println!("");

    match szun::decode(&out, &mut 0) {
        Ok(t) => match t {
            szun::Type::Schema(data) => {
                println!("len: {}", data.length());
                for i in 0..data.length() {
                    println!("'{}': {} ({:02x})", data.keyof(i).unwrap(), data.get(i).unwrap(), szun::kind(data.get(i).unwrap()));
                }
            }
            _ => { println!("not expected"); }
        }
        Err(_) => { println!("failed"); }
    }
}
