use crate::runtime::{
    Reference,
    acquire, release,
    type_inner, type_key,
    sparse_length,
    sparse_at, sparse_get,
    sparse_clear,
    sparse_set, sparse_unset,
    sparse_indexof,
};
use crate::tag;
use super::sparse;

pub struct Sparse {
    managed:bool,
    addr:Reference,
}
impl Sparse {
    pub fn new(class:usize) -> Self
    {
        Self {
            managed:true,
            addr:unsafe {acquire(sparse(class))},
        }
    }

    pub fn from(addr:Reference) -> Result<Self,()>
    {
        return if(unsafe {type_key(addr.class)} == tag::SPARSE) {
            Ok(Self { managed:false, addr:addr })
        }
        else {
            Err(())
        }
    }

    pub fn with(class:usize, data:Vec<(usize, Reference)>) -> Self
    {
        let mut obj = Self::new(class);
        for (key, item) in data {
            obj.set(key, item);
        }
        return obj;
    }

    pub fn length(&self) -> usize
    {
        unsafe {sparse_length(self.addr)}
    }

    pub fn at(&self, index:usize) -> Reference
    {
        unsafe {sparse_at(self.addr, index)}
    }

    pub fn has(&self, key:usize) -> bool
    {
        unsafe {sparse_get(self.addr, key)}.address != 0
    }

    pub fn get(&self, key:usize) -> Reference
    {
        unsafe {sparse_get(self.addr, key)}
    }

    pub fn clear(&mut self)
    {
        unsafe{sparse_clear(self.addr)};
    }

    pub fn set(&mut self, key:usize, source:Reference)
    {
        unsafe{sparse_set(self.addr, key, source)};
    }

    pub fn unset(&mut self, key:usize)
    {
        unsafe{sparse_unset(self.addr, key)};
    }

    pub fn indexof(&self, index:usize) -> usize
    {
        unsafe{sparse_indexof(self.addr, index)}
    }

    pub fn kind(&self) -> usize
    {
        unsafe {type_inner(self.addr.class)}
    }
}
impl std::ops::Deref for Sparse {
    type Target = Reference;
    fn deref(&self) -> &Self::Target { return &self.addr; }
}
impl Drop for Sparse {
    fn drop(&mut self) { if self.managed { unsafe {release(self.addr)}; } }
}
